#!/bin/python
# vim:et:ts=4

import smtplib, cgi, settings, whrandom, string, re
from AuthDB import AuthDB

db = AuthDB(settings.DB_PATH)
form = cgi.FieldStorage()

print """Content-type: text/html\r\n\r\n
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
        "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
    <title>IRC Authentication: """ + settings.NICK + """ on """ + settings.CHANNEL + """</title>
    <link rel="stylesheet" type="text/css" href="/style.css" />
</head>
<body>
<h1>IRC Authentication: """ + settings.NICK + """ on """ + settings.CHANNEL + """</h1>
"""

def value(key):
    """Get a CGI form value, or the empty string if not defined."""
    if form.has_key(key): return form[key].value
    return ""

mode = value("mode")
username = value("username")
newuser = value("newuser")
newpw1 = value("newpw1")
newpw2 = value("newpw2")

if username != "":
    password = value("password")
    modes = db.login(username, password)
    if modes is None: mode = "badpw"

if mode == "":
    print '''
<p>Welcome to the UKCIRC authentication service. The channel
''' + settings.CHANNEL + ''' is now auto-moderated; in order to talk on the
channel,
you must first identify yourself to the bot ''' + settings.NICK + ''' by typing
"/msg ''' + settings.NICK + ''' ident <i>&lt;username&gt; &lt;password&gt;</i>". To set up a username
and password, use this page. To find out which UKC user an authenticated nick corresponds to, type "/msg ''' + settings.NICK + ''' whois <i>&lt;nick&gt;</i>".</p>

<h2>Add a new IRC user</h2>

<form method="post" action="''' + settings.CGI_URL + '''">
<input type="hidden" name="mode" value="new" />
<p>UKC username (for instance, "xyz1"):
<input type="text" name="newuser" /></p>
</p>
<p><input type="submit" value="Mail me a password" /></p>
</form>

<h2>Change your password</h2>

<form method="post" action="''' + settings.CGI_URL + '''">
<input type="hidden" name="mode" value="changepw" />
<p>Remember: we cannot guarantee the security of your password here.
Do not use a password that you use elsewhere; in particular, do
not use your UKC password.</p>
<p>Username: <input type="text" name="username" /></p>
<p>Old password: <input type="password" name="password" /></p>
<p>New password: <input type="password" name="newpw1" /></p>
<p>Confirm new password: <input type="password" name="newpw2" /></p>
<p><input type="submit" value="Change password" /></p>
</form>
'''

elif mode == "new" and not re.compile("^[a-z][a-z]+[0-9]*$").match(newuser):
    print """<p>The username supplied is not a valid UKC username.
Please go back and correct it.</p>"""

elif mode == "new" and db.exists(newuser):
    print """<p>The specified username already exists. If you have lost
the password, contact """ + settings.MANAGER + """.</p>"""

elif mode == "new":
    username = value("newuser")
        
    # FIXME: sanity-check username
    mailaddr = username + "@ukc.ac.uk"

    password = ""
    for x in range(settings.PWLEN):
        password += whrandom.choice(string.letters).lower()

    server = smtplib.SMTP("localhost")
    msg = "From: " + settings.MANAGER + "\r\n"
    msg += "To: " + mailaddr + "\r\n"
    msg += "Subject: IRC authentication details\r\n"
    msg += "\r\n"
    msg += "This is an automated message from " + settings.NICK + " on " + settings.CHANNEL + ".\r\n"
    msg += "The password for " + username + " is \"" + password + "\".\r\n"
    msg += "You may now identify yourself to " + settings.NICK + " by typing:\r\n"
    msg += "  /msg " + settings.NICK + " ident " + username + " " + password + "\r\n"
    msg += "To change this, visit: " + settings.CGI_URL + "\r\n"
    msg += "\r\n"
    msg += "If you have received this message in error, please contact " + settings.MANAGER + ".\r\n"
    server.sendmail("ukcirc-auth@azz.us-lot.org", [mailaddr], msg)
    # print "<pre>" + msg + "</pre>"
    server.quit()

    db.set_password(username, password)
    db.set_modes(username, "+v")

    print """<p>Your password has been sent to """ + mailaddr + """.</p>"""

elif mode == "changepw" and newpw1 != newpw2:
    print """<p>New passwords do not match.
Please go back and correct them.</p>"""

elif mode == "changepw":
    db.set_password(username, newpw1)
    print """<p>Your password has been changed.</p>"""

elif mode == "badpw":
    print """
<p>Unable to authenticate your username and password.
Please go back and correct them.</p>
"""

else:
    print """<p>Unknown mode.</p>"""

print """
<p class="by">Please report any problems to
<a href="mailto:""" + settings.MANAGER + '''">''' + settings.MANAGER + """</a>.
</p>
</body></html>"""

